@@ -19,6 +19,10 @@ from numbers import Number
 from threading import Thread
 from time import time
 from time import sleep
+from contextlib import contextmanager
+# For parallelism safety
+import multiprocessing as mp
+import threading as th
 
 
 __author__ = {"github.com/": ["noamraph", "obiwanus", "kmike", "hadim",
@@ -44,6 +48,38 @@ class TqdmDeprecationWarning(Exception):
             super(TqdmDeprecationWarning, self).__init__(msg, *a, **k)
 
 
+# Create global parallelism locks to avoid racing issues with parallel bars
+# works only if fork available (Linux, MacOSX, but not on Windows)
+mp_lock = mp.Lock()  # multiprocessing lock
+th_lock = th.Lock()  # thread lock
+
+
+class TqdmDefaultWriteLock(object):
+    """
+    Provide a default write lock for thread and multiprocessing safety.
+    Works only on platforms supporting `fork` (so Windows is excluded).
+    On Windows, you need to supply the lock from the parent to the children as
+    an argument to joblib or the parallelism lib you use.
+    """
+    def __init__(self):
+        global mp_lock, th_lock
+        self.locks = [mp_lock, th_lock]
+
+    def acquire(self):
+        for lock in self.locks:
+            lock.acquire()
+
+    def release(self):
+        for lock in self.locks[::-1]:  # Release in inverse order of acquisition
+            lock.release()
+
+    def __enter__(self):
+        self.acquire()
+
+    def __exit__(self, *exc):
+        self.release()
+
+
 class TMonitor(Thread):
     """
     Monitoring thread for tqdm bars.
@@ -97,19 +133,21 @@ class TMonitor(Thread):
             if self.was_killed:
                 return
             # Then monitor!
-            cur_t = self._time()
-            # Check for each tqdm instance if one is waiting too long to print
-            # NB: copy avoids size change during iteration RuntimeError
-            for instance in self.tqdm_cls._instances.copy():
-                # Only if mininterval > 1 (else iterations are just slow)
-                # and last refresh was longer than maxinterval in this instance
-                if instance.miniters > 1 and \
-                        (cur_t - instance.last_print_t) >= instance.maxinterval:
-                    # We force bypassing miniters on next iteration
-                    # dynamic_miniters should adjust mininterval automatically
-                    instance.miniters = 1
-                    # Refresh now! (works only for manual tqdm)
-                    instance.refresh()
+            # Acquire lock (to access _instances)
+            with self.tqdm_cls.get_lock():
+                cur_t = self._time()
+                # Check tqdm instances are waiting too long to print
+                for instance in self.tqdm_cls._instances:
+                    # Only if mininterval > 1 (else iterations are just slow)
+                    # and last refresh exceeded maxinterval
+                    if instance.miniters > 1 and \
+                            (cur_t - instance.last_print_t) >= \
+                            instance.maxinterval:
+                        # force bypassing miniters on next iteration
+                        # (dynamic_miniters adjusts mininterval automatically)
+                        instance.miniters = 1
+                        # Refresh now! (works only for manual tqdm)
+                        instance.refresh(nolock=True)
 
     def report(self):
         # return self.is_alive()  # TODO: does not work...
@@ -197,12 +235,13 @@ class tqdm(object):
             len_s = len(s)
             fp_write('\r' + s + (' ' * max(last_len[0] - len_s, 0)))
             last_len[0] = len_s
+
         return print_status
 
     @staticmethod
-    def format_meter(n, total, elapsed, ncols=None, prefix='',
-                     ascii=False, unit='it', unit_scale=False, rate=None,
-                     bar_format=None, postfix=None, unit_divisor=1000):
+    def format_meter(n, total, elapsed, ncols=None, prefix='', ascii=False,
+                     unit='it', unit_scale=False, rate=None, bar_format=None,
+                     postfix=None, unit_divisor=1000):
         """
         Return a string-based progress bar given some parameters
 
@@ -223,6 +262,7 @@ class tqdm(object):
             will not print any meter (only stats).
         prefix  : str, optional
             Prefix message (included in total width) [default: ''].
+            Use as {desc} in bar_format string.
         ascii  : bool, optional
             If not set, use unicode (smooth blocks) to fill the meter
             [default: False]. The fallback is to use ASCII characters
@@ -239,13 +279,19 @@ class tqdm(object):
             If [default: None], uses n/elapsed.
         bar_format  : str, optional
             Specify a custom bar string formatting. May impact performance.
-            [default: '{l_bar}{bar}{r_bar}'], where l_bar is
-            '{desc}{percentage:3.0f}%|' and r_bar is
-            '| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
-            Possible vars: bar, n, n_fmt, total, total_fmt, percentage,
-            rate, rate_fmt, elapsed, remaining, l_bar, r_bar, desc.
+            [default: '{l_bar}{bar}{r_bar}'], where
+            l_bar='{desc}: {percentage:3.0f}%|' and
+            r_bar='| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, '
+                  '{rate_fmt}{postfix}]'
+            Possible vars: l_bar, bar, r_bar, n, n_fmt, total, total_fmt,
+                percentage, rate, rate_fmt, rate_noinv, rate_noinv_fmt,
+                rate_inv, rate_inv_fmt, elapsed, remaining, desc, postfix.
+            Note that a trailing ": " is automatically removed after {desc}
+            if the latter is empty.
         postfix  : str, optional
-            Same as prefix but will be placed at the end as additional stats.
+            Similar to `prefix`, but placed at the end
+            (e.g. for additional stats).
+            Note: postfix is a string for this method. Not a dict.
         unit_divisor  : float, optional
             [default: 1000], ignored unless `unit_scale` is True.
 
@@ -271,13 +317,15 @@ class tqdm(object):
         # (we allow manual override since predicting time is an arcane art)
         if rate is None and elapsed:
             rate = n / elapsed
-        inv_rate = 1 / rate if (rate and (rate < 1)) else None
+        inv_rate = 1 / rate if rate else None
         format_sizeof = tqdm.format_sizeof
-        rate_fmt = ((format_sizeof(inv_rate if inv_rate else rate)
-                    if unit_scale else
-                    '{0:5.2f}'.format(inv_rate if inv_rate else rate))
-                    if rate else '?') \
-            + ('s' if inv_rate else unit) + '/' + (unit if inv_rate else 's')
+        rate_noinv_fmt = ((format_sizeof(rate) if unit_scale else
+                           '{0:5.2f}'.format(rate))
+                          if rate else '?') + unit + '/s'
+        rate_inv_fmt = ((format_sizeof(inv_rate) if unit_scale else
+                         '{0:5.2f}'.format(inv_rate))
+                        if inv_rate else '?') + 's/' + unit
+        rate_fmt = rate_inv_fmt if inv_rate and inv_rate > 1 else rate_noinv_fmt
 
         if unit_scale:
             n_fmt = format_sizeof(n, divisor=unit_divisor)
@@ -297,11 +345,16 @@ class tqdm(object):
                 if rate else '?'
 
             # format the stats displayed to the left and right sides of the bar
-            l_bar = (prefix if prefix else '') + \
-                '{0:3.0f}%|'.format(percentage)
+            if prefix:
+                # old prefix setup work around
+                bool_prefix_colon_already = (prefix[-2:] == ": ")
+                l_bar = prefix if bool_prefix_colon_already else prefix + ": "
+            else:
+                l_bar = ''
+            l_bar += '{0:3.0f}%|'.format(percentage)
             r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
-                    n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt,
-                    ', ' + postfix if postfix else '')
+                n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt,
+                ', ' + postfix if postfix else '')
 
             if ncols == 0:
                 return l_bar[:-1] + r_bar[1:]
@@ -314,22 +367,26 @@ class tqdm(object):
                             'total': total,
                             'total_fmt': total_fmt,
                             'percentage': percentage,
-                            'rate': rate if inv_rate is None else inv_rate,
-                            'rate_noinv': rate,
-                            'rate_noinv_fmt': ((format_sizeof(rate)
-                                               if unit_scale else
-                                               '{0:5.2f}'.format(rate))
-                                               if rate else '?') + unit + '/s',
+                            'rate': inv_rate if inv_rate and inv_rate > 1
+                            else rate,
                             'rate_fmt': rate_fmt,
+                            'rate_noinv': rate,
+                            'rate_noinv_fmt': rate_noinv_fmt,
+                            'rate_inv': inv_rate,
+                            'rate_inv_fmt': rate_inv_fmt,
                             'elapsed': elapsed_str,
                             'remaining': remaining_str,
                             'l_bar': l_bar,
                             'r_bar': r_bar,
-                            'desc': prefix if prefix else '',
+                            'desc': prefix or '',
                             'postfix': ', ' + postfix if postfix else '',
                             # 'bar': full_bar  # replaced by procedure below
                             }
 
+                # auto-remove colon for empty `desc`
+                if not prefix:
+                    bar_format = bar_format.replace("{desc}: ", '')
+
                 # Interpolate supplied bar format with the dict
                 if '{bar}' in bar_format:
                     # Format left/right sides of the bar, and format the bar
@@ -375,20 +432,25 @@ class tqdm(object):
 
         # no total: no progressbar, ETA, just progress stats
         else:
-            return (prefix if prefix else '') + '{0}{1} [{2}, {3}{4}]'.format(
-                n_fmt, unit, elapsed_str, rate_fmt,
-                ', ' + postfix if postfix else '')
+            return ((prefix + ": ") if prefix else '') + \
+                '{0}{1} [{2}, {3}{4}]'.format(
+                    n_fmt, unit, elapsed_str, rate_fmt,
+                    ', ' + postfix if postfix else '')
 
     def __new__(cls, *args, **kwargs):
+        # Create default lock if none set
+        if "_lock" not in cls.__dict__:
+            cls.set_lock(TqdmDefaultWriteLock())
         # Create a new instance
         instance = object.__new__(cls)
         # Add to the list of instances
         if "_instances" not in cls.__dict__:
             cls._instances = WeakSet()
-        cls._instances.add(instance)
+        with cls._lock:
+            cls._instances.add(instance)
         # Create the monitoring thread
-        if cls.monitor_interval and (cls.monitor is None or
-                                     not cls.monitor.report()):
+        if cls.monitor_interval and (cls.monitor is None or not
+                                     cls.monitor.report()):
             cls.monitor = TMonitor(cls, cls.monitor_interval)
         # Return the instance
         return instance
@@ -411,48 +473,73 @@ class tqdm(object):
         so that newer bars won't overlap previous bars
         """
         try:  # in case instance was explicitly positioned, it won't be in set
-            cls._instances.remove(instance)
-            for inst in cls._instances:
-                if inst.pos > instance.pos:
-                    inst.pos -= 1
+            with cls._lock:
+                cls._instances.remove(instance)
+                for inst in cls._instances:
+                    if inst.pos > instance.pos:
+                        inst.pos -= 1
             # Kill monitor if no instances are left
             if not cls._instances and cls.monitor:
-                cls.monitor.exit()
                 try:
+                    cls.monitor.exit()
                     del cls.monitor
-                except AttributeError:
+                except AttributeError:  # pragma: nocover
                     pass
-                cls.monitor = None
+                else:
+                    cls.monitor = None
         except KeyError:
             pass
 
     @classmethod
-    def write(cls, s, file=None, end="\n"):
+    def write(cls, s, file=None, end="\n", nolock=False):
         """
         Print a message via tqdm (without overlap with bars)
         """
         fp = file if file is not None else sys.stdout
+        with cls.external_write_mode(file=file, nolock=nolock):
+            # Write the message
+            fp.write(s)
+            fp.write(end)
 
+    @classmethod
+    @contextmanager
+    def external_write_mode(cls, file=None, nolock=False):
+        """
+        Disable tqdm within context and refresh tqdm when exits.
+        Useful when writing to standard output stream
+        """
+        fp = file if file is not None else sys.stdout
+
+        if not nolock:
+            cls._lock.acquire()
         # Clear all bars
         inst_cleared = []
         for inst in getattr(cls, '_instances', []):
             # Clear instance if in the target output file
             # or if write output + tqdm output are both either
             # sys.stdout or sys.stderr (because both are mixed in terminal)
-            if inst.fp == fp or all(f in (sys.stdout, sys.stderr)
-                                    for f in (fp, inst.fp)):
-                inst.clear()
+            if inst.fp == fp or all(
+                    f in (sys.stdout, sys.stderr) for f in (fp, inst.fp)):
+                inst.clear(nolock=True)
                 inst_cleared.append(inst)
-        # Write the message
-        fp.write(s)
-        fp.write(end)
+        yield
         # Force refresh display of bars we cleared
         for inst in inst_cleared:
             # Avoid race conditions by checking that the instance started
             if hasattr(inst, 'start_t'):  # pragma: nocover
-                inst.refresh()
+                inst.refresh(nolock=True)
+        if not nolock:
+            cls._lock.release()
         # TODO: make list of all instances incl. absolutely positioned ones?
 
+    @classmethod
+    def set_lock(cls, lock):
+        cls._lock = lock
+
+    @classmethod
+    def get_lock(cls):
+        return cls._lock
+
     @classmethod
     def pandas(tclass, *targs, **tkwargs):
         """
@@ -541,6 +628,7 @@ class tqdm(object):
                 # Close bar and return pandas calculation result
                 t.close()
                 return result
+
             return inner
 
         # Monkeypatch pandas to provide easy methods
@@ -562,12 +650,11 @@ class tqdm(object):
         GroupBy.progress_transform = inner_generator('transform')
 
     def __init__(self, iterable=None, desc=None, total=None, leave=True,
-                 file=None, ncols=None, mininterval=0.1,
-                 maxinterval=10.0, miniters=None, ascii=None, disable=False,
-                 unit='it', unit_scale=False, dynamic_ncols=False,
-                 smoothing=0.3, bar_format=None, initial=0, position=None,
-                 postfix=None, unit_divisor=1000,
-                 gui=False, **kwargs):
+                 file=None, ncols=None, mininterval=0.1, maxinterval=10.0,
+                 miniters=None, ascii=None, disable=False, unit='it',
+                 unit_scale=False, dynamic_ncols=False, smoothing=0.3,
+                 bar_format=None, initial=0, position=None, postfix=None,
+                 unit_divisor=1000, gui=False, **kwargs):
         """
         Parameters
         ----------
@@ -636,10 +723,12 @@ class tqdm(object):
         bar_format  : str, optional
             Specify a custom bar string formatting. May impact performance.
             If unspecified, will use '{l_bar}{bar}{r_bar}', where l_bar is
-            '{desc}{percentage:3.0f}%|' and r_bar is
+            '{desc}: {percentage:3.0f}%|' and r_bar is
             '| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
             Possible vars: bar, n, n_fmt, total, total_fmt, percentage,
             rate, rate_fmt, elapsed, remaining, l_bar, r_bar, desc.
+            Note that a trailing ": " is automatically removed after {desc}
+            if the latter is empty.
         initial  : int, optional
             The initial counter value. Useful when restarting a progress
             bar [default: 0].
@@ -649,6 +738,8 @@ class tqdm(object):
             Useful to manage multiple bars at once (eg, from threads).
         postfix  : dict, optional
             Specify additional stats to display at the end of the bar.
+            Note: postfix is a dict ({'key': value} pairs) for this method,
+            not a string.
         unit_divisor  : float, optional
             [default: 1000], ignored unless `unit_scale` is True.
         gui  : bool, optional
@@ -672,6 +763,7 @@ class tqdm(object):
             self.disable = disable
             self.pos = self._get_free_pos(self)
             self._instances.remove(self)
+            self.n = initial
             return
 
         if kwargs:
@@ -680,9 +772,8 @@ class tqdm(object):
             self._instances.remove(self)
             raise (TqdmDeprecationWarning("""\
 `nested` is deprecated and automated. Use position instead for manual control.
-""", fp_write=getattr(file, 'write', sys.stderr.write))
-                if "nested" in kwargs else
-                TqdmKeyError("Unknown argument(s): " + str(kwargs)))
+""", fp_write=getattr(file, 'write', sys.stderr.write)) if "nested" in kwargs
+                else TqdmKeyError("Unknown argument(s): " + str(kwargs)))
 
         # Preprocess the arguments
         if total is None and iterable is not None:
@@ -730,7 +821,7 @@ class tqdm(object):
 
         # Store the arguments
         self.iterable = iterable
-        self.desc = desc + ': ' if desc else ''
+        self.desc = desc or ''
         self.total = total
         self.leave = leave
         self.fp = file
@@ -752,7 +843,7 @@ class tqdm(object):
         self.bar_format = bar_format
         self.postfix = None
         if postfix:
-            self.set_postfix(**postfix)
+            self.set_postfix(refresh=False, **postfix)
 
         # Init the iterations counters
         self.last_print_n = initial
@@ -769,14 +860,12 @@ class tqdm(object):
         if not gui:
             # Initialize the screen printer
             self.sp = self.status_printer(self.fp)
-            if self.pos:
-                self.moveto(self.pos)
-            self.sp(self.format_meter(self.n, total, 0,
-                    (dynamic_ncols(file) if dynamic_ncols else ncols),
-                    self.desc, ascii, unit, unit_scale, None, bar_format,
-                    self.postfix, unit_divisor))
-            if self.pos:
-                self.moveto(-self.pos)
+            with self._lock:
+                if self.pos:
+                    self.moveto(self.pos)
+                self.sp(self.__repr__(elapsed=0))
+                if self.pos:
+                    self.moveto(-self.pos)
 
         # Init the time counter
         self.last_print_t = self._time()
@@ -799,15 +888,14 @@ class tqdm(object):
     def __del__(self):
         self.close()
 
-    def __repr__(self):
-        return self.format_meter(self.n, self.total,
-                                 self._time() - self.start_t,
-                                 self.dynamic_ncols(self.fp)
-                                 if self.dynamic_ncols else self.ncols,
-                                 self.desc, self.ascii, self.unit,
-                                 self.unit_scale, 1 / self.avg_time
-                                 if self.avg_time else None, self.bar_format,
-                                 self.postfix)
+    def __repr__(self, elapsed=None):
+        return self.format_meter(
+            self.n, self.total,
+            elapsed if elapsed is not None else self._time() - self.start_t,
+            self.dynamic_ncols(self.fp) if self.dynamic_ncols else self.ncols,
+            self.desc, self.ascii, self.unit,
+            self.unit_scale, 1 / self.avg_time if self.avg_time else None,
+            self.bar_format, self.postfix, self.unit_divisor)
 
     def __lt__(self, other):
         return self.pos < other.pos
@@ -842,25 +930,16 @@ class tqdm(object):
             for obj in iterable:
                 yield obj
         else:
-            ncols = self.ncols
             mininterval = self.mininterval
             maxinterval = self.maxinterval
             miniters = self.miniters
             dynamic_miniters = self.dynamic_miniters
-            unit = self.unit
-            unit_scale = self.unit_scale
-            unit_divisor = self.unit_divisor
-            ascii = self.ascii
-            start_t = self.start_t
             last_print_t = self.last_print_t
             last_print_n = self.last_print_n
             n = self.n
-            dynamic_ncols = self.dynamic_ncols
             smoothing = self.smoothing
             avg_time = self.avg_time
-            bar_format = self.bar_format
             _time = self._time
-            format_meter = self.format_meter
 
             try:
                 sp = self.sp
@@ -871,17 +950,16 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
             for obj in iterable:
                 yield obj
-                # Update and print the progressbar.
+                # Update and possibly print the progressbar.
                 # Note: does not call self.update(1) for speed optimisation.
                 n += 1
-                # check the counter first (avoid calls to time())
+                # check counter first to avoid calls to time()
                 if n - last_print_n >= self.miniters:
                     miniters = self.miniters  # watch monitoring thread changes
                     delta_t = _time() - last_print_t
                     if delta_t >= mininterval:
                         cur_t = _time()
                         delta_it = n - last_print_n
-                        elapsed = cur_t - start_t  # optimised if in inner loop
                         # EMA (not just overall average)
                         if smoothing and delta_t and delta_it:
                             avg_time = delta_t / delta_it \
@@ -889,20 +967,14 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                                 else smoothing * delta_t / delta_it + \
                                 (1 - smoothing) * avg_time
 
-                        if self.pos:
-                            self.moveto(self.pos)
-
-                        # Printing the bar's update
-                        sp(format_meter(
-                            n, self.total, elapsed,
-                            (dynamic_ncols(self.fp) if dynamic_ncols
-                             else ncols),
-                            self.desc, ascii, unit, unit_scale,
-                            1 / avg_time if avg_time else None, bar_format,
-                            self.postfix, unit_divisor))
-
-                        if self.pos:
-                            self.moveto(-self.pos)
+                        self.n = n
+                        with self._lock:
+                            if self.pos:
+                                self.moveto(self.pos)
+                            # Print bar update
+                            sp(self.__repr__())
+                            if self.pos:
+                                self.moveto(-self.pos)
 
                         # If no `miniters` was specified, adjust automatically
                         # to the max iteration rate seen so far between 2 prints
@@ -920,8 +992,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                                 # towards the timeframe of mininterval
                                 miniters = smoothing * delta_it * \
                                     (mininterval / delta_t
-                                     if mininterval and delta_t
-                                     else 1) + \
+                                     if mininterval and delta_t else 1) + \
                                     (1 - smoothing) * miniters
                             else:
                                 # Maximum nb of iterations between 2 prints
@@ -955,10 +1026,11 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
         Parameters
         ----------
-        n  : int
+        n  : int, optional
             Increment to add to the internal counter of iterations
             [default: 1].
         """
+        # N.B.: see __iter__() for more comments.
         if self.disable:
             return
 
@@ -966,12 +1038,12 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
             raise ValueError("n ({0}) cannot be negative".format(n))
         self.n += n
 
+        # check counter first to reduce calls to time()
         if self.n - self.last_print_n >= self.miniters:
-            # We check the counter first, to reduce the overhead of time()
             delta_t = self._time() - self.last_print_t
             if delta_t >= self.mininterval:
                 cur_t = self._time()
-                delta_it = self.n - self.last_print_n  # should be n?
+                delta_it = self.n - self.last_print_n  # >= n
                 # elapsed = cur_t - self.start_t
                 # EMA (not just overall average)
                 if self.smoothing and delta_t and delta_it:
@@ -985,14 +1057,15 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 """, fp_write=getattr(self.fp, 'write', sys.stderr.write))
 
-                if self.pos:
-                    self.moveto(self.pos)
+                with self._lock:
+                    if self.pos:
+                        self.moveto(self.pos)
 
-                # Print bar's update
-                self.sp(self.__repr__())
+                    # Print bar update
+                    self.sp(self.__repr__())
 
-                if self.pos:
-                    self.moveto(-self.pos)
+                    if self.pos:
+                        self.moveto(-self.pos)
 
                 # If no `miniters` was specified, adjust automatically to the
                 # maximum iteration rate seen so far between two prints.
@@ -1049,29 +1122,25 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                 return
             raise  # pragma: no cover
 
-        if pos:
-            self.moveto(pos)
+        with self._lock:
+            if pos:
+                self.moveto(pos)
 
-        if self.leave:
-            if self.last_print_n < self.n:
-                cur_t = self._time()
-                # stats for overall rate (no weighted average)
-                self.sp(self.format_meter(
-                    self.n, self.total, cur_t - self.start_t,
-                    (self.dynamic_ncols(self.fp) if self.dynamic_ncols
-                     else self.ncols),
-                    self.desc, self.ascii, self.unit, self.unit_scale, None,
-                    self.bar_format, self.postfix, self.unit_divisor))
-            if pos:
-                self.moveto(-pos)
+            if self.leave:
+                if self.last_print_n < self.n:
+                    # stats for overall rate (no weighted average)
+                    self.avg_time = None
+                    self.sp(self.__repr__())
+                if pos:
+                    self.moveto(-pos)
+                else:
+                    fp_write('\n')
             else:
-                fp_write('\n')
-        else:
-            self.sp('')  # clear up last bar
-            if pos:
-                self.moveto(-pos)
-            else:
-                fp_write('\r')
+                self.sp('')  # clear up last bar
+                if pos:
+                    self.moveto(-pos)
+                else:
+                    fp_write('\r')
 
     def unpause(self):
         """
@@ -1081,16 +1150,39 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
         self.start_t += cur_t - self.last_print_t
         self.last_print_t = cur_t
 
-    def set_description(self, desc=None):
+    def set_description(self, desc=None, refresh=True):
         """
         Set/modify description of the progress bar.
+
+        Parameters
+        ----------
+        desc  : str, optional
+        refresh  : bool, optional
+            Forces refresh [default: True].
         """
         self.desc = desc + ': ' if desc else ''
+        if refresh:
+            self.refresh()
 
-    def set_postfix(self, ordered_dict=None, **kwargs):
+    def set_description_str(self, desc=None, refresh=True):
+        """
+        Set/modify description without ': ' appended.
+        """
+        self.desc = desc or ''
+        if refresh:
+            self.refresh()
+
+    def set_postfix(self, ordered_dict=None, refresh=True, **kwargs):
         """
         Set/modify postfix (additional stats)
         with automatic formatting based on datatype.
+
+        Parameters
+        ----------
+        ordered_dict  : dict or OrderedDict, optional
+        refresh  : bool, optional
+            Forces refresh [default: True].
+        kwargs  : dict, optional
         """
         # Sort in alphabetical order to be more deterministic
         postfix = _OrderedDict([] if ordered_dict is None else ordered_dict)
@@ -1108,17 +1200,29 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
         # Stitch together to get the final postfix
         self.postfix = ', '.join(key + '=' + postfix[key].strip()
                                  for key in postfix.keys())
+        if refresh:
+            self.refresh()
+
+    def set_postfix_str(self, s='', refresh=True):
+        """
+        Postfix without dictionary expansion, similar to prefix handling.
+        """
+        self.postfix = str(s)
+        if refresh:
+            self.refresh()
 
     def moveto(self, n):
         self.fp.write(_unicode('\n' * n + _term_move_up() * -n))
 
-    def clear(self, nomove=False):
+    def clear(self, nomove=False, nolock=False):
         """
         Clear current bar display
         """
         if self.disable:
             return
 
+        if not nolock:
+            self._lock.acquire()
         if not nomove:
             self.moveto(self.pos)
         # clear up the bar (can't rely on sp(''))
@@ -1127,20 +1231,26 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
         self.fp.write('\r')  # place cursor back at the beginning of line
         if not nomove:
             self.moveto(-self.pos)
+        if not nolock:
+            self._lock.release()
 
-    def refresh(self):
+    def refresh(self, nolock=False):
         """
         Force refresh the display of this bar
         """
         if self.disable:
             return
 
+        if not nolock:
+            self._lock.acquire()
         self.moveto(self.pos)
         # clear up this line's content (whatever there was)
-        self.clear(nomove=True)
+        self.clear(nomove=True, nolock=True)
         # Print current/last bar state
         self.fp.write(self.__repr__())
         self.moveto(-self.pos)
+        if not nolock:
+            self._lock.release()
 
 
 def trange(*args, **kwargs):
